/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_serv.c (formerly ircd/s_msg.c)
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include <string.h>
#include "h.h"

static char buf[BUFSIZE];


/*
** m_functions execute protocol messages on this server:
**
**	cptr	is always NON-NULL, pointing to a *LOCAL* client
**		structure (with an open socket connected!). This
**		identifies the physical socket where the message
**		originated (or which caused the m_function to be
**		executed--some m_functions may call others...).
**
**	sptr	is the source of the message, defined by the
**		prefix part of the message if present. If not
**		or prefix not found, then sptr==cptr.
**
**		(!IsServer(cptr)) => (cptr == sptr), because
**		prefixes are taken *only* from servers...
**
**		(IsServer(cptr))
**			(sptr == cptr) => the message didn't
**			have the prefix.
**
**			(sptr != cptr && IsServer(sptr) means
**			the prefix specified servername. (?)
**
**			(sptr != cptr && !IsServer(sptr) means
**			that message originated from a remote
**			user (not local).
**
**		combining
**
**		(!IsServer(sptr)) means that, sptr can safely
**		taken as defining the target structure of the
**		message in this server.
**	*Always* true (if 'parse' and others are working correct):
**
**	1)	sptr->from == cptr  (note: cptr->from == cptr)
**
**	2)	MyConnect(sptr) <=> sptr == cptr (e.g. sptr
**		*cannot* be a local connection, unless it's
**		actually cptr!). [MyConnect(x) should probably
**		be defined as (x == x->from) --msa ]
**
**	parc	number of variable parameter strings (if zero,
**		parv is allowed to be NULL)
**
**	parv	a NULL terminated list of parameter pointers,
**
**			parv[0], sender (prefix string), if not present
**				this points to an empty string.
**			parv[1]...parv[parc-1]
**				pointers to additional parameters
**			parv[parc] == NULL, *always*
**
**		note:	it is guaranteed that parv[0]..parv[parc-1] are all
**			non-NULL pointers.
*/

/*
** m_users
**	parv[0] = sender prefix
**	parv[1] = servername
*/
int
m_users (cptr, sptr, parc, parv)
     aClient *cptr, *sptr;
     int parc;
     char *parv[];
{
#ifdef ENABLE_USERS
  char namebuf[10], linebuf[10], hostbuf[17];
  int fd, flag = 0;
#endif

  if (check_registered_user (sptr))
    return 0;

  if (hunt_server (cptr, sptr, ":%s USERS :%s", 1, parc, parv) == HUNTED_ISME)
    {
#ifdef ENABLE_USERS
      if ((fd = utmp_open ()) == -1)
	{
	  sendto_one (sptr, err_str (ERR_FILEERROR),
		      me.name, parv[0], "open", UTMP);
	  return 0;
	}

      sendto_one (sptr, rpl_str (RPL_USERSSTART), me.name, parv[0]);
      while (utmp_read (fd, namebuf, linebuf, hostbuf, sizeof (hostbuf)) == 0)
	{
	  flag = 1;
	  sendto_one (sptr, rpl_str (RPL_USERS), me.name, parv[0],
		      namebuf, linebuf, hostbuf);
	}
      if (flag == 0)
	sendto_one (sptr, rpl_str (RPL_NOUSERS), me.name, parv[0]);

      sendto_one (sptr, rpl_str (RPL_ENDOFUSERS), me.name, parv[0]);
      (void) utmp_close (fd);
#else
      sendto_one (sptr, err_str (ERR_USERSDISABLED), me.name, parv[0]);
#endif
    }
  return 0;
}
